@@ -28,22 +28,25 @@ import qualified Data.ByteString as S
 import qualified Data.ByteString.Char8 as SC
 import qualified Data.ByteString.Lazy as L
 import           Data.ByteString.Internal (c2w, w2c)
-import qualified Data.ByteString.Nums.Careless.Int as Cvt
 import           Data.Enumerator.Internal
 import           Data.Int
 import           Data.IORef
 import           Data.List (foldl')
 import qualified Data.Map as Map
-import           Data.Maybe (catMaybes, fromJust, fromMaybe, isJust)
+import           Data.Maybe ( catMaybes, fromJust, fromMaybe, isJust
+                            , isNothing )
 import           Data.Monoid
+import qualified Data.Text as T
+import qualified Data.Text.Encoding as T
 import           Data.Time
 import           Data.Typeable
 import           Data.Version
 import           GHC.Conc
-import           Network.Socket (withSocketsDo)
+import           Network.Socket (withSocketsDo, Socket)
+#if !MIN_VERSION_base(4,6,0)
 import           Prelude hiding (catch)
-import           System.PosixCompat.Files hiding (setFileSize)
-import           System.Posix.Types (FileOffset)
+#endif
+import           System.IO
 import           System.Locale
 ------------------------------------------------------------------------------
 import           System.FastLogger (timestampedLogEntry, combinedLogEntry)
@@ -57,9 +60,9 @@ import           Snap.Internal.Http.Server.Backend
 import           Snap.Internal.Http.Server.HttpPort
 import qualified Snap.Internal.Http.Server.TLS as TLS
 import           Snap.Internal.Http.Server.SimpleBackend
-import           Snap.Internal.Http.Server.LibevBackend
 
 import           Snap.Internal.Iteratee.Debug
+import           Snap.Internal.Parsing (unsafeFromInt)
 import           Snap.Iteratee hiding (head, take, map)
 import qualified Snap.Iteratee as I
 
@@ -103,12 +106,6 @@ instance Show ListenPort where
         concat [ "https://", SC.unpack b, ":", show p, "/" ]
 
 
-------------------------------------------------------------------------------
-data EventLoopType = EventLoopSimple
-                   | EventLoopLibEv
-  deriving (Show)
-
-
 ------------------------------------------------------------------------------
 -- This exception will be thrown if we decided to terminate the request before
 -- running the user handler.
@@ -123,15 +120,6 @@ data ExceptionAlreadyCaught = ExceptionAlreadyCaught
 instance Exception ExceptionAlreadyCaught
 
 
-------------------------------------------------------------------------------
-defaultEvType :: EventLoopType
-#ifdef LIBEV
-defaultEvType = EventLoopLibEv
-#else
-defaultEvType = EventLoopSimple
-#endif
-
-
 ------------------------------------------------------------------------------
 data ServerState = ServerState
     { _forceConnectionClose  :: Bool
@@ -161,23 +149,51 @@ runServerMonad lh s la le m = evalStateT m st
 ------------------------------------------------------------------------------
 httpServe :: Int                         -- ^ default timeout
           -> [ListenPort]                -- ^ ports to listen on
-          -> Maybe EventLoopType         -- ^ Specify a given event loop,
-                                         --   otherwise a default is picked
           -> ByteString                  -- ^ local hostname (server name)
           -> Maybe (ByteString -> IO ()) -- ^ access log action
           -> Maybe (ByteString -> IO ()) -- ^ error log action
+          -> ([Socket] -> IO ())         -- ^ initialisation
           -> ServerHandler               -- ^ handler procedure
           -> IO ()
-httpServe defaultTimeout ports mevType localHostname alog' elog'
-          handler = withSocketsDo $ spawnAll alog' elog'
+httpServe defaultTimeout ports localHostname alog' elog' initial handler =
+    withSocketsDo $ spawnAll alog' elog' `catches` errorHandlers
+
   where
+    --------------------------------------------------------------------------
+    errorHandlers = [ Handler sslException
+                    , Handler threadWasKilled
+                    , Handler otherException ]
+
+    --------------------------------------------------------------------------
+    sslException (e :: TLS.TLSException) = do
+        let msg = SC.concat [
+                    "This version of snap-server was not built with SSL "
+                  , "support.\n"
+                  , "Please compile snap-server with -fopenssl to enable it."
+                  ]
+
+        logE elog' msg
+        SC.hPutStrLn stderr msg
+        throw e
+
+    ------------------------------------------------------------------------------
+    threadWasKilled (_ :: AsyncException) = return ()
+
+    ------------------------------------------------------------------------------
+    otherException (e :: SomeException) = do
+        let msg = SC.concat [
+                    "Error on startup: \n"
+                  , T.encodeUtf8 $ T.pack $ show e
+                  ]
+        logE elog' msg
+        SC.hPutStrLn stderr msg
+        throw e
+
     --------------------------------------------------------------------------
     spawnAll alog elog = {-# SCC "httpServe/spawnAll" #-} do
 
-        let evType = maybe defaultEvType id mevType
-
-        logE elog $ S.concat [ "Server.httpServe: START ("
-                             , toBS $ show evType, ")"]
+        logE elog $ S.concat [ "Server.httpServe: START, binding to "
+                             , bshow ports ]
 
         let isHttps p = case p of { (HttpsPort _ _ _ _) -> True; _ -> False;}
         let initHttps = foldr (\p b -> b || isHttps p) False ports
@@ -187,8 +203,9 @@ httpServe defaultTimeout ports mevType localHostname alog' elog'
             else return ()
 
         nports <- mapM bindPort ports
+        let socks = map (\x -> case x of ListenHttp s -> s; ListenHttps s _ -> s) nports
 
-        (runEventLoop evType defaultTimeout nports numCapabilities (logE elog)
+        (simpleEventLoop defaultTimeout nports numCapabilities (logE elog) (initial socks)
                     $ runHTTP defaultTimeout alog elog handler localHostname)
           `finally` do
             logE elog "Server.httpServe: SHUTDOWN"
@@ -205,11 +222,6 @@ httpServe defaultTimeout ports mevType localHostname alog' elog'
         TLS.bindHttps baddr port cert key
 
 
-    --------------------------------------------------------------------------
-    runEventLoop EventLoopSimple       = simpleEventLoop
-    runEventLoop EventLoopLibEv        = libEvEventLoop
-
-
 ------------------------------------------------------------------------------
 debugE :: (MonadIO m) => ByteString -> m ()
 debugE s = debug $ "Server: " ++ (map w2c $ S.unpack s)
@@ -419,7 +431,7 @@ httpSession defaultTimeout writeEnd' buffer onSendFile tickle handler = do
                        then id
                        else insHeader
           let rsp' = updateHeaders ins rsp
-          (bytesSent,_) <- sendResponse req rsp' buffer writeEnd onSendFile
+          (bytesSent,_) <- sendResponse rsp' buffer writeEnd onSendFile
                            `catch` errCatch "sending response" req
 
           debug $ "Server.httpSession: sent " ++
@@ -588,7 +600,7 @@ receiveRequest writeEnd = do
 
 
         hdrs = rqHeaders req
-        mbCL = H.lookup "content-length" hdrs >>= return . Cvt.int . head
+        mbCL = H.lookup "content-length" hdrs >>= return . unsafeFromInt . head
 
 
     --------------------------------------------------------------------------
@@ -682,7 +694,7 @@ receiveRequest writeEnd = do
 
         hdrs            = toHeaders kvps
 
-        mbContentLength = liftM (Cvt.int . head) $
+        mbContentLength = liftM (unsafeFromInt . head) $
                           H.lookup "content-length" hdrs
 
         cookies         = concat $
@@ -692,7 +704,7 @@ receiveRequest writeEnd = do
 
         contextPath     = "/"
 
-        parseHost h = (a, Cvt.int (S.drop 1 b))
+        parseHost h = (a, unsafeFromInt (S.drop 1 b))
           where
             (a,b) = S.break (== (c2w ':')) h
 
@@ -704,16 +716,21 @@ receiveRequest writeEnd = do
 
 ------------------------------------------------------------------------------
 -- Response must be well-formed here
-sendResponse :: forall a . Request
-             -> Response
+sendResponse :: forall a . Response
              -> Buffer
              -> Iteratee ByteString IO a             -- ^ iteratee write end
              -> (FilePath -> Int64 -> Int64 -> IO a) -- ^ function to call on
                                                      -- sendfile
              -> ServerMonad (Int64, a)
-sendResponse req rsp' buffer writeEnd' onSendFile = do
-    let rsp'' = renderCookies rsp'
-    rsp <- fixupResponse rsp''
+sendResponse rsp0 buffer writeEnd' onSendFile = do
+    let rsp1 = renderCookies rsp0
+
+    let (rsp, shouldClose) = if isNothing $ rspContentLength rsp1
+                               then noCL rsp1
+                               else (rsp1, False)
+
+    when shouldClose $ modify $! \s -> s { _forceConnectionClose = True }
+
     let (!headerString,!hlen) = mkHeaderBuilder rsp
     let writeEnd = fixCLIteratee hlen rsp writeEnd'
 
@@ -731,6 +748,32 @@ sendResponse req rsp' buffer writeEnd' onSendFile = do
     return $! (bs,x)
 
   where
+    --------------------------------------------------------------------------
+    noCL :: Response -> (Response, Bool)
+    noCL r =
+        if rspHttpVersion r >= (1,1)
+          then
+            let r'    = setHeader "Transfer-Encoding" "chunked" r
+                origE = rspBodyToEnum $ rspBody r
+                e     = \i -> joinI $ origE $$ chunkIt i
+            in (r' { rspBody = Enum e }, False)
+        else
+           -- HTTP/1.0 and no content-length? We'll have to close the
+           -- socket.
+           (setHeader "Connection" "close" r, True)
+    {-# INLINE noCL #-}
+
+
+    --------------------------------------------------------------------------
+    chunkIt :: forall x . Enumeratee Builder Builder IO x
+    chunkIt = checkDone $ continue . step
+      where
+        step k EOF = k (Chunks [chunkedTransferTerminator]) >>== return
+        step k (Chunks []) = continue $ step k
+        step k (Chunks xs) = k (Chunks [chunkedTransferEncoding $ mconcat xs])
+                             >>== chunkIt
+
+
     --------------------------------------------------------------------------
     whenEnum :: Iteratee ByteString IO a
              -> Builder
@@ -792,7 +835,7 @@ sendResponse req rsp' buffer writeEnd' onSendFile = do
 
 
     --------------------------------------------------------------------------
-    (major,minor) = rspHttpVersion rsp'
+    (major,minor) = rspHttpVersion rsp0
 
 
     --------------------------------------------------------------------------
@@ -820,36 +863,6 @@ sendResponse req rsp' buffer writeEnd' onSendFile = do
             klen    = S.length k' + 2
 
 
-    --------------------------------------------------------------------------
-    noCL :: Response
-         -> ServerMonad Response
-    noCL r = {-# SCC "noCL" #-} do
-        -- are we in HTTP/1.1?
-        let sendChunked = (rspHttpVersion r) == (1,1)
-        if sendChunked
-          then do
-              let r' = setHeader "Transfer-Encoding" "chunked" r
-              let origE = rspBodyToEnum $ rspBody r
-
-              let e = \i -> joinI $ origE $$ chunkIt i
-
-              return $! r' { rspBody = Enum e }
-
-          else do
-              -- HTTP/1.0 and no content-length? We'll have to close the
-              -- socket.
-              modify $! \s -> s { _forceConnectionClose = True }
-              return $! setHeader "Connection" "close" r
-
-    --------------------------------------------------------------------------
-    chunkIt :: forall x . Enumeratee Builder Builder IO x
-    chunkIt = checkDone $ continue . step
-      where
-        step k EOF = k (Chunks [chunkedTransferTerminator]) >>== return
-        step k (Chunks []) = continue $ step k
-        step k (Chunks xs) = k (Chunks [chunkedTransferEncoding $ mconcat xs])
-                             >>== chunkIt
-
     --------------------------------------------------------------------------
     fixCLIteratee :: Int                       -- ^ header length
                   -> Response                  -- ^ response
@@ -864,30 +877,6 @@ sendResponse req rsp' buffer writeEnd' onSendFile = do
 
         mbCL = rspContentLength resp
 
-    --------------------------------------------------------------------------
-    hasCL :: Int64
-          -> Response
-          -> ServerMonad Response
-    hasCL cl r = {-# SCC "hasCL" #-}
-        -- set the content-length header
-        return $! setHeader "Content-Length" (toByteString $ fromShow cl) r
-
-
-    --------------------------------------------------------------------------
-    setFileSize :: FilePath -> Response -> ServerMonad Response
-    setFileSize fp r =
-        {-# SCC "setFileSize" #-}
-        do
-            fs <- liftM fromIntegral $ liftIO $ getFileSize fp
-            return $! r { rspContentLength = Just fs }
-
-
-    --------------------------------------------------------------------------
-    handle304 :: Response -> Response
-    handle304 r = setResponseBody (enumBuilder mempty) $
-                  updateHeaders (H.delete "Transfer-Encoding") $
-                  setContentLength 0 r
-
 
     --------------------------------------------------------------------------
     renderCookies :: Response -> Response
@@ -899,34 +888,6 @@ sendResponse req rsp' buffer writeEnd' onSendFile = do
         cookies = fmap cookieToBS . Map.elems $ rspCookies r
 
 
-    --------------------------------------------------------------------------
-    fixupResponse :: Response
-                  -> ServerMonad Response
-    fixupResponse r = {-# SCC "fixupResponse" #-} do
-        let r' = deleteHeader "Content-Length" r
-        let code = rspStatus r'
-        let r'' = if code == 204 || code == 304
-                   then handle304 r'
-                   else r'
-
-        r''' <- do
-            z <- case rspBody r'' of
-                   (Enum _)                  -> return r''
-                   (SendFile f Nothing)      -> setFileSize f r''
-                   (SendFile _ (Just (s,e))) -> return $!
-                                                setContentLength (e-s) r''
-
-            case rspContentLength z of
-              Nothing   -> noCL z
-              (Just sz) -> hasCL sz z
-
-        -- HEAD requests cannot have bodies per RFC 2616 sec. 9.4
-        if rqMethod req == HEAD
-          then return $! deleteHeader "Transfer-Encoding" $
-                         r''' { rspBody = Enum $ enumBuilder mempty }
-          else return $! r'''
-
-
     --------------------------------------------------------------------------
     mkHeaderBuilder :: Response -> (Builder,Int)
     mkHeaderBuilder r = {-# SCC "mkHeaderBuilder" #-}
@@ -996,11 +957,6 @@ cookieToBS (Cookie k v mbExpTime mbDomain mbPath isSec isHOnly) = cookie
                                    "%a, %d-%b-%Y %H:%M:%S GMT"
 
 
-------------------------------------------------------------------------------
-getFileSize :: FilePath -> IO FileOffset
-getFileSize fp = liftM fileSize $ getFileStatus fp
-
-
 ------------------------------------------------------------------------------
 l2s :: L.ByteString -> S.ByteString
 l2s = S.concat . L.toChunks
