@@ -26,7 +26,9 @@ import           Foreign hiding (new)
 import           Foreign.C
 import           GHC.Conc (labelThread, forkOnIO)
 import           Network.Socket
+#if !MIN_VERSION_base(4,6,0)
 import           Prelude hiding (catch)
+#endif
 ------------------------------------------------------------------------------
 import           Snap.Internal.Debug
 import           Snap.Internal.Http.Server.Date
@@ -59,10 +61,11 @@ data EventLoopCpu = EventLoopCpu
 
 ------------------------------------------------------------------------------
 simpleEventLoop :: EventLoop
-simpleEventLoop defaultTimeout sockets cap elog handler = do
+simpleEventLoop defaultTimeout sockets cap elog initial handler = do
     loops <- Prelude.mapM (newLoop defaultTimeout sockets handler elog)
                           [0..(cap-1)]
 
+    initial
     debug "simpleEventLoop: waiting for mvars"
 
     --wait for all threads to exit
