@@ -1,9 +1,9 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE BangPatterns        #-}
+{-# LANGUAGE CPP                 #-}
+{-# LANGUAGE DeriveDataTypeable  #-}
+{-# LANGUAGE OverloadedStrings   #-}
+{-# LANGUAGE PackageImports      #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE PackageImports #-}
-{-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE BangPatterns #-}
 
 module Snap.Internal.Http.Server.Tests
   ( tests ) where
@@ -11,51 +11,47 @@ module Snap.Internal.Http.Server.Tests
 import           Blaze.ByteString.Builder
 import           Blaze.ByteString.Builder.Enumerator
 import           Control.Concurrent
-import           Control.Exception ( catch
-                                   , try
-                                   , throwIO
-                                   , bracket
-                                   , finally
-                                   , SomeException
-                                   , Exception )
+import           Control.Exception                   (Exception, SomeException,
+                                                      bracket, catch, finally,
+                                                      throwIO, try)
 import           Control.Monad
 import           Control.Monad.Trans
-import qualified Data.ByteString.Char8 as S
-import qualified Data.ByteString.Lazy as L
-import qualified Data.ByteString.Lazy.Char8 as LC
-import           Data.ByteString (ByteString)
-import           Data.ByteString.Internal (c2w)
-import qualified Data.CaseInsensitive as CI
+import           Data.ByteString                     (ByteString)
+import qualified Data.ByteString.Char8               as S
+import           Data.ByteString.Internal            (c2w)
+import qualified Data.ByteString.Lazy                as L
+import qualified Data.ByteString.Lazy.Char8          as LC
+import qualified Data.CaseInsensitive                as CI
 import           Data.Char
-import qualified Data.Enumerator as E
-import qualified Data.Enumerator.List as EL
+import qualified Data.Enumerator                     as E
+import qualified Data.Enumerator.List                as EL
 import           Data.Int
 import           Data.IORef
-import           Data.List (foldl', sort)
-import           Data.Maybe (fromJust, isJust)
+import           Data.List                           (foldl', sort)
+import           Data.Maybe                          (fromJust, isJust)
 import           Data.Time.Calendar
 import           Data.Time.Clock
 import           Data.Typeable
-import qualified Network.HTTP.Enumerator as HTTP
-import qualified Network.Socket.ByteString as N
-import           Prelude hiding (catch, take)
+import qualified Network.HTTP.Conduit                as HTTP
+import qualified Network.Socket.ByteString           as N
+import           Prelude                             hiding (catch, take)
 import qualified Prelude
 import           System.Timeout
 import           Test.Framework
 import           Test.Framework.Providers.HUnit
-import           Test.HUnit hiding (Test, path)
+import           Test.HUnit                          hiding (Test, path)
 
-import qualified Snap.Http.Server as Svr
+import qualified Snap.Http.Server                    as Svr
 
 import           Snap.Core
 import           Snap.Internal.Debug
-import           Snap.Internal.Http.Types
 import           Snap.Internal.Http.Server
-import qualified Snap.Iteratee as I
-import           Snap.Iteratee hiding (map)
 import           Snap.Internal.Http.Server.Backend
+import           Snap.Internal.Http.Types
+import           Snap.Iteratee                       hiding (map)
+import qualified Snap.Iteratee                       as I
 import           Snap.Test.Common
-import qualified Snap.Types.Headers as H
+import qualified Snap.Types.Headers                  as H
 
 data TestException = TestException
   deriving (Show, Typeable)
@@ -152,20 +148,14 @@ testMethodParsing :: Test
 testMethodParsing =
     testCase "server/method parsing" $ Prelude.mapM_ testOneMethod ms
   where
-    ms = [ GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS, CONNECT ]
+    ms = [ GET, HEAD, POST, PUT, DELETE, TRACE, OPTIONS, CONNECT, PATCH
+         , Method "COPY", Method "MOVE" ]
 
 
 dummyIter :: Iteratee ByteString IO ()
 dummyIter = consume >> return ()
 
 
-mkRequest :: ByteString -> IO Request
-mkRequest s = do
-    step <- runIteratee $ liftM fromJust $ rsm $ receiveRequest dummyIter
-    let iter = enumBS s step
-    run_ iter
-
-
 testReceiveRequest :: Iteratee ByteString IO (Request,L.ByteString)
 testReceiveRequest = do
     r  <- liftM fromJust $ rsm $ receiveRequest dummyIter
@@ -272,9 +262,11 @@ testPartialParse = testCase "server/short" $ do
 
 
 methodTestText :: Method -> L.ByteString
-methodTestText m = L.concat [ (L.pack $ map c2w $ show m)
+methodTestText m = L.concat [ mbs m
                         , " / HTTP/1.1\r\nContent-Length: 0\r\n\r\n" ]
-
+  where
+    mbs (Method b) = L.fromChunks [b]
+    mbs b = L.pack $ map c2w $ show b
 
 sampleRequest2 :: ByteString
 sampleRequest2 =
@@ -380,11 +372,10 @@ rsm = runServerMonad "localhost" (SessionInfo "127.0.0.1" 80 "127.0.0.1" 58382 F
 
 testHttpResponse1 :: Test
 testHttpResponse1 = testCase "server/HttpResponse1" $ do
-    req   <- mkRequest sampleRequest
     buf   <- allocBuffer 16384
 
     b     <- run_ $ rsm $
-             sendResponse req rsp1 buf copyingStream2Stream testOnSendFile >>=
+             sendResponse rsp1 buf copyingStream2Stream testOnSendFile >>=
                           return . snd
 
     assertBool "http response" (b == text1 || b == text2)
@@ -403,7 +394,7 @@ testHttpResponse1 = testCase "server/HttpResponse1" $ do
                      ]
 
     rsp1 = updateHeaders (H.insert "Foo" "Bar") $
-           setContentLength 10 $
+           setContentLength' 10 $
            setResponseStatus 600 "Test" $
            modifyResponseBody (>==> (enumBuilder $
                                      fromByteString "0123456789")) $
@@ -432,10 +423,9 @@ testOnSendFile f st sz = do
 
 testHttpResponse2 :: Test
 testHttpResponse2 = testCase "server/HttpResponse2" $ do
-    req   <- mkRequest sampleRequest
     buf   <- allocBuffer 16384
     b2    <- liftM (S.concat . L.toChunks) $ run_ $ rsm $
-             sendResponse req rsp2 buf copyingStream2Stream testOnSendFile >>=
+             sendResponse rsp2 buf copyingStream2Stream testOnSendFile >>=
                           return . snd
 
     assertBool "http prefix"
@@ -452,7 +442,7 @@ testHttpResponse2 = testCase "server/HttpResponse2" $ do
 
   where
     rsp1 = updateHeaders (H.insert "Foo" "Bar") $
-           setContentLength 10 $
+           setContentLength' 10 $
            setResponseStatus 600 "Test" $
            modifyResponseBody (>==> (enumBuilder $
                                      fromByteString "0123456789")) $
@@ -463,11 +453,10 @@ testHttpResponse2 = testCase "server/HttpResponse2" $ do
 
 testHttpResponse3 :: Test
 testHttpResponse3 = testCase "server/HttpResponse3" $ do
-    req   <- mkRequest sampleRequest
     buf   <- allocBuffer 16384
 
     b3 <- run_ $ rsm $
-          sendResponse req rsp3 buf copyingStream2Stream testOnSendFile >>=
+          sendResponse rsp3 buf copyingStream2Stream testOnSendFile >>=
                        return . snd
 
     let lns = LC.lines b3
@@ -495,13 +484,13 @@ testHttpResponse3 = testCase "server/HttpResponse3" $ do
         hdrs = strToHeaders s
 
     rsp1 = updateHeaders (H.insert "Foo" "Bar") $
-           setContentLength 10 $
+           setContentLength' 10 $
            setResponseStatus 600 "Test" $
            modifyResponseBody (>==> (enumBuilder $
                                      fromByteString "0123456789")) $
            setResponseBody returnI $
            emptyResponse { rspHttpVersion = (1,0) }
-    rsp2 = rsp1 { rspContentLength = Nothing }
+    rsp2 = deleteHeader "Content-Length" $ rsp1 { rspContentLength = Nothing }
     rsp3 = setContentType "text/plain" $ (rsp2 { rspHttpVersion = (1,1) })
 
 
@@ -509,10 +498,8 @@ testHttpResponse4 :: Test
 testHttpResponse4 = testCase "server/HttpResponse4" $ do
     buf   <- allocBuffer 16384
 
-    req <- mkRequest sampleRequest
-
     b <- run_ $ rsm $
-         sendResponse req rsp1 buf copyingStream2Stream testOnSendFile >>=
+         sendResponse rsp1 buf copyingStream2Stream testOnSendFile >>=
                       return . snd
 
     assertEqual "http response" (L.concat [
@@ -522,16 +509,15 @@ testHttpResponse4 = testCase "server/HttpResponse4" $ do
 
   where
     rsp1 = setResponseStatus 304 "Test" $
+           setContentLength' 0 $
            emptyResponse { rspHttpVersion = (1,0) }
 
 
 testHttpResponseCookies :: Test
 testHttpResponseCookies = testCase "server/HttpResponseCookies" $ do
     buf <- allocBuffer 16384
-    req <- mkRequest sampleRequest
-
     b <- run_ $ rsm $
-          sendResponse req rsp2 buf copyingStream2Stream testOnSendFile >>=
+          sendResponse rsp2 buf copyingStream2Stream testOnSendFile >>=
                       return . snd
 
     let lns = LC.lines b
@@ -547,7 +533,7 @@ testHttpResponseCookies = testCase "server/HttpResponseCookies" $ do
     assertBool "http response" ok
 
   where
-    check s = (H.lookup "Content-Length" hdrs == Just ["0\r"]) &&
+    check s = (H.lookup "Connection" hdrs == Just ["close\r"]) &&
               (ch $ H.lookup "Set-Cookie" hdrs)
       where
         hdrs = strToHeaders s
@@ -586,9 +572,9 @@ echoServer _ _ req = do
     liftIO $ writeIORef (rqBody req) (SomeEnumerator $ joinI . I.take 0)
     return (req, rsp b cl)
   where
-    rsp s cl = emptyResponse { rspBody = Enum $
-                                         enumBuilder (fromLazyByteString s)
-                             , rspContentLength = Just $ fromIntegral cl }
+    rsp s cl = setContentLength' cl $
+               emptyResponse { rspBody = Enum $
+                                         enumBuilder (fromLazyByteString s) }
 
 
 echoServer2 :: ServerHandler
@@ -925,7 +911,7 @@ testExpectGarbage = testCase "server/expectGarbage" $ do
 pongServer :: Snap ()
 pongServer = modifyResponse $ setResponseBody enum .
                               setContentType "text/plain" .
-                              setContentLength 4
+                              setContentLength' 4
   where
     enum = enumBuilder $ fromByteString "PONG"
 
@@ -944,8 +930,8 @@ testSendFile = testCase "server/sendFile" $ do
                        m)
 
   where
-    serve = (httpServe 60 [HttpPort "*" port] Nothing "localhost"
-                       Nothing Nothing
+    serve = (httpServe 60 [HttpPort "*" port] "localhost"
+                       Nothing Nothing (const $ return ())
                     $ runSnap sendFileFoo)
             `catch` \(_::SomeException) -> return ()
 
@@ -970,10 +956,10 @@ testServerStartupShutdown = testCase "server/startup/shutdown" $ do
     bracket (forkIO $
              httpServe 20
                        [HttpPort "*" port]
-                       Nothing
                        "localhost"
                        (Just $ const (return ())) -- dummy logging
                        (Just $ const (return ())) -- dummy logging
+                       (const $ return ())
                        (runSnap pongServer))
             (killThread)
             (\tid -> do
@@ -1008,11 +994,11 @@ testServerShutdownWithOpenConns :: Test
 testServerShutdownWithOpenConns = testCase "server/shutdown-open-conns" $ do
     tid <- forkIO $
            httpServe 20
-                     [HttpPort "127.0.0.1" port]
-                     Nothing
+                     [HttpPort "*" port]
                      "localhost"
                      Nothing
                      Nothing
+                     (const $ return ())
                      (runSnap pongServer)
 
     waitabit
@@ -1050,7 +1036,7 @@ testServerShutdownWithOpenConns = testCase "server/shutdown-open-conns" $ do
 
   where
     waitabit = threadDelay $ 2*((10::Int)^(6::Int))
-    port = 8146
+    port = 8149
 
 
 
@@ -1066,3 +1052,8 @@ copyingStream2Stream = go []
         maybe (return $ L.fromChunks $ reverse l)
               (\x -> let !z = S.copy x in go (z:l))
               mbx
+
+
+setContentLength' :: Int64 -> Response -> Response
+setContentLength' cl = setHeader "Content-Length" (S.pack $ show cl) .
+                       setContentLength cl
