@@ -1,7 +1,7 @@
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE BangPatterns        #-}
+{-# LANGUAGE OverloadedStrings   #-}
+{-# LANGUAGE PackageImports      #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE PackageImports #-}
-{-# LANGUAGE BangPatterns #-}
 
 module Test.Blackbox
   ( tests
@@ -11,36 +11,38 @@ module Test.Blackbox
 --------------------------------------------------------------------------------
 import           Blaze.ByteString.Builder
 import           Control.Concurrent
-import           Control.Exception (SomeException, catch, throwIO)
+import           Control.Exception                    (SomeException, catch,
+                                                       throwIO)
 import           Control.Monad
 import           Control.Monad.Trans
-import qualified Data.ByteString.Base16 as B16
-import qualified Data.ByteString.Char8 as S
-import           Data.ByteString.Char8 (ByteString)
-import qualified Data.ByteString.Lazy.Char8 as L
-import           Data.CaseInsensitive (CI)
+import qualified Data.ByteString.Base16               as B16
+import           Data.ByteString.Char8                (ByteString)
+import qualified Data.ByteString.Char8                as S
+import qualified Data.ByteString.Lazy.Char8           as L
+import           Data.CaseInsensitive                 (CI)
+import           Data.Conduit                         (ResourceT)
 import           Data.Int
 import           Data.List
 import           Data.Monoid
-import qualified Network.HTTP.Enumerator as HTTP
-import qualified Network.Socket.ByteString as N
-import           Network.TLS (TLSCertificateUsage(..))
-import           Prelude hiding (catch, take)
+import qualified Network.HTTP.Conduit                 as HTTP
+import qualified Network.Socket.ByteString            as N
+import           Network.TLS                          (CertificateUsage (..))
+import           Prelude                              hiding (catch, take)
 import           System.Timeout
 import           Test.Framework
 import           Test.Framework.Options
 import           Test.Framework.Providers.HUnit
 import           Test.Framework.Providers.QuickCheck2
-import           Test.HUnit hiding (Test, path)
+import           Test.HUnit                           hiding (Test, path)
 import           Test.QuickCheck
-import qualified Test.QuickCheck.Property as QC
-import qualified Test.QuickCheck.Monadic as QC
-import           Test.QuickCheck.Monadic hiding (run, assert)
+import           Test.QuickCheck.Monadic              hiding (assert, run)
+import qualified Test.QuickCheck.Monadic              as QC
+import qualified Test.QuickCheck.Property             as QC
 ------------------------------------------------------------------------------
-import           Snap.Internal.Debug
-import           Snap.Iteratee hiding (map, head)
-import qualified Snap.Iteratee as I
 import           Snap.Http.Server
+import           Snap.Internal.Debug
+import           Snap.Iteratee                        hiding (head, map)
+import qualified Snap.Iteratee                        as I
 import           Snap.Test.Common
 import           Test.Common.Rot13
 import           Test.Common.TestHandler
@@ -64,8 +66,8 @@ testFunctions = [ testPong
 
 
 ------------------------------------------------------------------------------
-tests :: Int -> String -> [Test]
-tests port name = map (\f -> f False port name) testFunctions
+tests :: Int -> [Test]
+tests port = map (\f -> f False port "") testFunctions
 
 
 ------------------------------------------------------------------------------
@@ -77,35 +79,34 @@ slowTestOptions ssl =
 
 
 ------------------------------------------------------------------------------
-ssltests :: String -> Maybe Int -> [Test]
-ssltests name = maybe [] httpsTests
+ssltests :: Maybe Int -> [Test]
+ssltests = maybe [] httpsTests
     where httpsTests port = map (\f -> f True port sslname) testFunctions
-          sslname = "ssl/" ++ name
+          sslname = "ssl/"
 
 ------------------------------------------------------------------------------
 startTestServer :: Int
                 -> Maybe Int
-                -> ConfigBackend
                 -> IO (ThreadId, MVar ())
-startTestServer port sslport backend = do
-    let cfg = setAccessLog
-                  (ConfigFileLog $ "ts-access." ++ show backend ++ ".log") .
-              setErrorLog
-                  (ConfigFileLog $ "ts-error." ++ show backend ++ ".log")  .
-              setBind      "*"                                             .
-              setPort      port                                            .
-              setBackend   backend                                         .
-              setDefaultTimeout 10                                         .
-              setVerbose   False                                           $
+startTestServer port sslport = do
+    let cfg = setAccessLog      (ConfigFileLog "ts-access.log") .
+              setErrorLog       (ConfigFileLog "ts-error.log")  .
+              setBind           "*"                             .
+              setPort           port                            .
+              setDefaultTimeout 10                              .
+              setVerbose        False                           $
               defaultConfig
 
-    let cfg' = case sslport of
-                Nothing -> cfg
-                Just p  -> setSSLPort p          .
-                           setSSLBind "*"        .
-                           setSSLCert "cert.pem" .
-                           setSSLKey  "key.pem"  $
-                           cfg
+    let cfg' = maybe cfg
+                     (\p ->
+                      setSSLPort   p                                   .
+                      setSSLBind   "*"                                 .
+                      setSSLCert   "cert.pem"                          .
+                      setSSLKey    "key.pem"                           .
+                      setAccessLog (ConfigFileLog "ts-access-ssl.log") .
+                      setErrorLog  (ConfigFileLog "ts-error-ssl.log")  $
+                      cfg)
+                     sslport
 
     mvar <- newEmptyMVar
     tid  <- forkIO $ do
@@ -146,7 +147,7 @@ doPong ssl port = do
 
 ------------------------------------------------------------------------------
 testPong :: Bool -> Int -> String -> Test
-testPong ssl port name = testCase (name ++ "/blackbox/pong") $ do
+testPong ssl port name = testCase (name ++ "blackbox/pong") $ do
     doc <- doPong ssl port
     assertEqual "pong response" "PONG" doc
 
@@ -154,7 +155,7 @@ testPong ssl port name = testCase (name ++ "/blackbox/pong") $ do
 ------------------------------------------------------------------------------
 -- FIXME: waiting on http-enumerator patch for HEAD behaviour
 -- testHeadPong :: Bool -> Int -> String -> Test
--- testHeadPong ssl port name = testCase (name ++ "/blackbox/pong/HEAD") $ do
+-- testHeadPong ssl port name = testCase (name ++ "blackbox/pong/HEAD") $ do
 --     doc <- headPong ssl port
 --     assertEqual "pong HEAD response" "" doc
 
@@ -163,7 +164,7 @@ testPong ssl port name = testCase (name ++ "/blackbox/pong") $ do
 testEcho :: Bool -> Int -> String -> Test
 testEcho ssl port name =
     plusTestOptions (slowTestOptions ssl) $
-    testProperty (name ++ "/blackbox/echo") $
+    testProperty (name ++ "blackbox/echo") $
     QC.mapSize (if ssl then min 100 else min 300) $
     monadicIO $ forAllM arbitrary prop
   where
@@ -179,7 +180,7 @@ testEcho ssl port name =
 testFileUpload :: Bool -> Int -> String -> Test
 testFileUpload ssl port name =
     plusTestOptions (slowTestOptions ssl) $
-    testProperty (name ++ "/blackbox/upload") $
+    testProperty (name ++ "blackbox/upload") $
     QC.mapSize (if ssl then min 100 else min 300) $
     monadicIO $
     forAllM arbitrary prop
@@ -249,7 +250,7 @@ testFileUpload ssl port name =
 testRot13 :: Bool -> Int -> String -> Test
 testRot13 ssl port name =
     plusTestOptions (slowTestOptions ssl) $
-    testProperty (name ++ "/blackbox/rot13") $
+    testProperty (name ++ "blackbox/rot13") $
     monadicIO $ forAllM arbitrary prop
   where
     prop txt = do
@@ -264,7 +265,7 @@ testRot13 ssl port name =
 ------------------------------------------------------------------------------
 -- TODO: this one doesn't work w/ SSL
 testSlowLoris :: Bool -> Int -> String -> Test
-testSlowLoris ssl port name = testCase (name ++ "/blackbox/slowloris") $
+testSlowLoris ssl port name = testCase (name ++ "blackbox/slowloris") $
                               if ssl then return () else withSock port go
 
   where
@@ -294,7 +295,7 @@ testSlowLoris ssl port name = testCase (name ++ "/blackbox/slowloris") $
 -- TODO: doesn't work w/ ssl
 testBlockingRead :: Bool -> Int -> String -> Test
 testBlockingRead ssl port name =
-    testCase (name ++ "/blackbox/testBlockingRead") $
+    testCase (name ++ "blackbox/testBlockingRead") $
              if ssl then return () else runIt
 
   where
@@ -324,7 +325,7 @@ testBlockingRead ssl port name =
 -- test server's ability to trap/recover from IO errors
 testPartial :: Bool -> Int -> String -> Test
 testPartial ssl port name =
-    testCase (name ++ "/blackbox/testPartial") $
+    testCase (name ++ "blackbox/testPartial") $
     if ssl then return () else runIt
 
   where
@@ -346,7 +347,7 @@ testPartial ssl port name =
 -- TODO: no ssl
 testBigResponse :: Bool -> Int -> String -> Test
 testBigResponse ssl port name =
-    testCase (name ++ "/blackbox/testBigResponse") $
+    testCase (name ++ "blackbox/testBigResponse") $
     if ssl then return () else runIt
   where
     runIt = withSock port $ \sock -> do
@@ -380,42 +381,38 @@ seconds = (10::Int) ^ (6::Int)
 
 
 ------------------------------------------------------------------------------
-parseURL :: String -> IO (HTTP.Request IO)
-parseURL url = do
-    req <- HTTP.parseUrl url
-    return $ req { HTTP.checkCerts = const $ const $
-                                     return CertificateUsageAccept }
-
-
-------------------------------------------------------------------------------
-fetchReq :: HTTP.Request IO -> IO (L.ByteString)
+fetchReq :: HTTP.Request (ResourceT IO) -> IO (L.ByteString)
 fetchReq req = go `catch` (\(e::SomeException) -> do
                  debug $ "simpleHttp threw exception: " ++ show e
                  throwIO e)
   where
     go = do
-        rsp <- HTTP.withManager $ HTTP.httpLbs req
+        rsp <- HTTP.withManagerSettings settings $ HTTP.httpLbs req
         return $ HTTP.responseBody rsp
 
+    settings = HTTP.def { HTTP.managerCheckCerts = \_ _ _ ->
+                          return CertificateUsageAccept }
 
 ------------------------------------------------------------------------------
-fetchResponse :: String -> IO (HTTP.Response)
+fetchResponse :: String -> IO (HTTP.Response L.ByteString)
 fetchResponse url = do
-    req <- parseURL url `catch` (\(e::SomeException) -> do
-               debug $ "parseURL threw exception: " ++ show e
+    req <- HTTP.parseUrl url `catch` (\(e::SomeException) -> do
+               debug $ "parseUrl threw exception: " ++ show e
                throwIO e)
     go req `catch` (\(e::SomeException) -> do
                        debug $ "simpleHttp threw exception: " ++ show e
                        throwIO e)
   where
-    go req = HTTP.withManager $ HTTP.httpLbs req
+    go req = HTTP.withManagerSettings settings $ HTTP.httpLbs req
+    settings = HTTP.def { HTTP.managerCheckCerts = \_ _ _ ->
+                          return CertificateUsageAccept }
 
 
 ------------------------------------------------------------------------------
 fetch :: String -> IO (L.ByteString)
 fetch url = do
-    req <- parseURL url `catch` (\(e::SomeException) -> do
-                 debug $ "parseURL threw exception: " ++ show e
+    req <- HTTP.parseUrl url `catch` (\(e::SomeException) -> do
+                 debug $ "HTTP.parseUrl threw exception: " ++ show e
                  throwIO e)
     fetchReq req
 
@@ -426,8 +423,8 @@ post :: String
      -> [(CI ByteString, ByteString)]
      -> IO (L.ByteString)
 post url body hdrs = do
-    req <- parseURL url `catch` (\(e::SomeException) -> do
-                 debug $ "parseURL threw exception: " ++ show e
+    req <- HTTP.parseUrl url `catch` (\(e::SomeException) -> do
+                 debug $ "HTTP.parseUrl threw exception: " ++ show e
                  throwIO e)
     fetchReq $ req { HTTP.requestBody    = HTTP.RequestBodyLBS body
                    , HTTP.method         = "POST"
@@ -441,7 +438,7 @@ post url body hdrs = do
 -- 2. that "flush" is passed along through a gzip operation.
 testTimeoutTickle :: Bool -> Int -> String -> Test
 testTimeoutTickle ssl port name =
-    testCase (name ++ "/blackbox/timeout/tickle") $ do
+    testCase (name ++ "blackbox/timeout/tickle") $ do
         let uri = (if ssl then "https" else "http")
                   ++ "://127.0.0.1:" ++ show port ++ "/timeout/tickle"
         doc <- liftM (S.concat . L.toChunks) $ fetch uri
@@ -452,7 +449,7 @@ testTimeoutTickle ssl port name =
 ------------------------------------------------------------------------------
 testTimeoutBadTickle :: Bool -> Int -> String -> Test
 testTimeoutBadTickle ssl port name =
-    testCase (name ++ "/blackbox/timeout/badtickle") $ do
+    testCase (name ++ "blackbox/timeout/badtickle") $ do
         let uri = (if ssl then "https" else "http")
                   ++ "://127.0.0.1:" ++ show port ++ "/timeout/badtickle"
         expectException $ fetch uri
